package examples.servletserver;

import java.net.*;
import java.io.*;

public class HttpServer {
  public static void main(String[] arguments) {
    int port = 8888;
    
    ServerSocket server_socket = null; // dummy initialization
    try {
      server_socket = new ServerSocket(port);
    } catch (IOException ioe) {
      System.out.println("Failed to bind to port "+port+": "+ioe.getMessage());
      System.exit(1);
    }
    
    while (true) {
      Socket connection_socket = null;
      try {
        connection_socket = server_socket.accept(); 
      } catch (IOException ioe) {
        System.out.println("Accept failed: "+ioe.getMessage());
        System.exit(1);
      }
    
      BufferedReader in = null;
      String servlet    = null;
      java.util.Map<String,String> parameters = null;
      
      try {
        in = new BufferedReader(
                              new InputStreamReader(connection_socket.getInputStream()));
        String line;
        
        while (true) {
          line = in.readLine();
          if (line==null)         break;
          if (line.length() == 0) break;
          System.out.println(line);
          
          if (line.startsWith("GET ")) {
            String get_params[] = line.split(" ");
            String path = get_params[1];
            //System.out.println("get path = "+path);
            servlet = path.substring(1);
            int index_of_question = servlet.indexOf('?');
            int index_of_slash    = servlet.indexOf('/');
            //System.out.println("index of q = "+index_of_question);
            parameters = new java.util.TreeMap<String,String>();
            if (index_of_question != -1) {
              String params = servlet.substring(index_of_question+1); 
              //System.out.println("params = <"+params+">");
              String[] params_values = params.split("&");
              for (String pv: params_values) {
                String[] p_and_v=pv.split("=");
                parameters.put(p_and_v[0], 
                               p_and_v.length==2 ? p_and_v[1] : "");
 
                servlet = servlet.substring(0,index_of_question);
             }
            } else if (index_of_slash != -1) {
              String path_param = servlet.substring(index_of_slash+1); 
              parameters.put("path",path_param);
              servlet = servlet.substring(0,index_of_slash);
            }
            
            System.out.println("servlet = "+servlet);
            for (String s: parameters.keySet()) {
              System.out.println("parameter <"+s+">=<"+parameters.get(s)+">");
            }
          }
        }
      } catch (IOException ioe) {
        System.out.println("Could not read the request: "+ioe.getMessage());
        System.exit(1);
      }
      
      PrintWriter out = null;
      try {
        out = new PrintWriter(connection_socket.getOutputStream());
        
       } catch (IOException ioe) {
        System.out.println("Could not send the output: "+ioe.getMessage());
        System.exit(1);
      }
       
      HttpServlet s = HttpServlet.getInstance(servlet);
      
      if (s != null) {
        ServletInvocation invocation = new ServletInvocation(parameters,out);
        s.doGet(invocation,invocation);        
      } else {
        out.print("HTTP/1.1 404\r\n");
        out.print("Content-Type: text/html\r\n");
        out.print("Connection: close\r\n");
        out.print("\r\n"); // end of header
        out.println("<HTML><HEAD>");
        out.println("<TITLE>404 Not Found</TITLE>");
        out.println("</HEAD><BODY>");
        out.println("<H1>Not Found</H1>");
        out.println("<P>The requested URL was not found on this server.</P>");
        out.println("</BODY></HTML>");
      }
      
      try {
        out.close();
        in.close();
        connection_socket.close();
      } catch (IOException ioe) {
        // who cares?
      }
    }
  }
}
